% Non-numerical strategies used by bees to solve numerical cognition tasks
% HaDi MaBouDi, Andrew B Barron, Sun Li, Maria Honkanen, Olli Loukola, Fei Peng, Wenfeng Li, 
% James A. R. Marshall, Alex Cope, Eleni Vasilaki, and Cwyn Solvi
% University of Sheffield
% h.maboudi@sheffield.ac.uk
% Dec 2020

% this code run the model of relational rule between the spatial frequnecy
% between stimuli with different number of elements

% this contains different sections that exhibits the model of different
% experiments discussed in 
%Howard SR, Avargus-Weber A, Garcia JE, Greentree AD, Dyer AG. 2018 Numerical ordering of zero in honey bees. Science 360, 1124?1126. (doi:10.1126/science.aar4975)
%%

%%
clear all; close all; clc

%% Neuronal tuning curve of sensory neurons 
% this section show the tuning curve of 7 neurons peak at 0, 1,... and 6
% spatial frequency. 

f=1:.1:7;

figure
for k=1:7
plot(f,TC(f,k,50,100,1),'-','color',(.2+.1*k)*[1 1 1],'LineWidth',2)
hold on
end
xlim([0 8])
%ylim([0 120])

ylabel('Firing rate (Spike/sec)')
xlabel('Spatial Freqnecy')
set(gca,'xTick',[1 2 3 4 5 6 7],'xTickLabel',{'0','1','2','3','4','5','6'})
set(gca, 'FontSize', 20)

legend('Neuron 1','Neuron 2','Neuron 3','Neuron 4','Neuron 5','Neuron 6','Neuron 7')

%% Activation function 
%/ this section shows two samples of activation function with different
% sensetivity 

G =@ (x,a,b) 100./(1+exp(-a*(x-b)));

x=-100:.1:200;
figure
plot(x,G(x,.04,70),'LineWidth',2)
hold on 
plot(x,G(x,.02,70),'LineWidth',2,'Color','k')

ylabel('Neuron Input ')
xlabel('Neuron Output')
set(gca, 'FontSize', 20)

%%   Experiment 1 (Fewer-than)  (Figure 2B)
% trained to [1 4] and testes
% this senction, the model is trained to different paired of stimuli
% between 1 and 4. The synaptic weights will be rewarded if the first pair
% is smaller than the second pair.
%%% the model then will be examined with pairs (0,3) & (2,5)


a=.06;
b=70; % change the baseline actvity of the post neurons
r0=40;
rm=100;
sigma=2.8;

% Sigmoid function
G =@ (x,a,b) 110./(1+exp(-a*(x-b)))+4*randn;
Gp=@ (x,a,b) 110*a*exp(-a*(x-b))./(1+exp(-a*(x-b))).^2;  

f=1:7;
eta=1e-3; % Learning term
n_trials=50; 
 
 
%%%%%%%%%%%%%%%%%%%%%%
%%%% "Less than" experiment 

% Training protocol
Stimulipair=[ 1 1 1 2 2 3 ;...
              2 3 4 3 4 4 ];    
                 
n_presentation=10;
StimuliPair=[];
for tNP=1:n_presentation
StimuliPair=[StimuliPair,Stimulipair];
end

 
for k=1:n_trials
% Initial synaptic weights 
W=rand(1,7);
W=W/sum(abs(W));

 stimuli2=StimuliPair(:,randperm(size(StimuliPair,2)));   

for m=1:size(stimuli2,2)
    kp=stimuli2(1,m)+1;
    kn=stimuli2(2,m)+1;
   
 dw=eta*(SensResp(kp,r0,rm,sigma)*Gp(sum(SensResp(kp,r0,rm,sigma).*W),a,b)-SensResp(kn,r0,rm,sigma)*Gp(sum(SensResp(kn,r0,rm,sigma).*W),a,b)); % best 
 
 W=W+dw;
  %W=W/sum(abs(W));
  
[k,m]
end

for t=1:7
   outneuron(t,k)= G(sum(SensResp(t,r0,rm,sigma).*W),a,b);
end


W_Updated(k,:)=W;
% 3 Vs 5
for f=1:10
LearningTest1(f)=(G(sum(SensResp(4,r0,rm,sigma).*W),a,b))-(G(sum(SensResp(5,r0,rm,sigma).*W),a,b));%/(sum(TC(f,2,r0,rm,sigma).*W)-sum(TC(f,6,r0,rm,sigma).*W)); % Test 3 vs 4 
% 0 vs 2
ConflictTest1(f)=(G(sum(SensResp(1,r0,rm,sigma).*W),a,b))-(G(sum(SensResp(4,r0,rm,sigma).*W),a,b));%/(sum(TC(f,2,r0,rm,sigma).*W)-sum(TC(f,6,r0,rm,sigma).*W)); % Test 0 vs 3 
% 0 vs 1
TransferTest1(f)=(G(sum(SensResp(3,r0,rm,sigma).*W),a,b))-(G(sum(SensResp(6,r0,rm,sigma).*W),a,b));%/(sum(TC(f,2,r0,rm,sigma).*W)-sum(TC(f,6,r0,rm,sigma).*W)); % Test 2 vs 5 
end

LearningTest_e1_1(k)=sum(LearningTest1>0)/10;
ConflictTest_e1_1(k)=sum(ConflictTest1>0)/10;
TransferTest_e1_1(k)=sum(TransferTest1>0)/10;

%W_updated(k,:)=W;
end

figure
subplot(1,3,1)
errorbar(mean(outneuron'),std(outneuron')/sqrt(n_trials))

subplot(1,3,3)
meanTest_Exp1_1=[mean(LearningTest_e1_1) mean(ConflictTest_e1_1) mean(TransferTest_e1_1)];
SETest_Exp1_1=[std(LearningTest_e1_1) std(ConflictTest_e1_1) std(TransferTest_e1_1)]/sqrt(n_trials);
hold on
bar(meanTest_Exp1_1)
errorbar(meanTest_Exp1_1,SETest_Exp1_1)
set(gca,'xTick',[1 2 3],'xTickLabel',{'Learning test[3 4]','Conflict test[0 3]','Transfer test[2 5]'})
ylabel('Performance (%)')
title('Model Performance in Tests')

subplot(1,3,2)
errorbar(mean(W_Updated),std(W_Updated)/sqrt(n_trials))
set(gca,'xTick',[1 2 3 4 5 6 7],'xTickLabel',{'0','1','2','3','4','5','6'})
title('optimal Synapic weights')


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% "Greater than" experiment

% Training protocol
Stimulipair=[ 1 1 1 2 2 3 ;...
              2 3 4 3 4 4 ];    
                 
n_presentation=10;
StimuliPair=[];
for tNP=1:n_presentation
StimuliPair=[StimuliPair,Stimulipair];
end
 
for k=1:n_trials
% Initial synaptic weights 
W=rand(1,7);
W=W/sum(abs(W));

 stimuli2=StimuliPair(:,randperm(size(StimuliPair,2)));   

for m=1:size(stimuli2,2)
    kp=stimuli2(2,m)+1;
    kn=stimuli2(1,m)+1;
   
 dw=eta*(SensResp(kp,r0,rm,sigma)*Gp(sum(SensResp(kp,r0,rm,sigma).*W),a,b)-SensResp(kn,r0,rm,sigma)*Gp(sum(SensResp(kn,r0,rm,sigma).*W),a,b)); % best 
 
 W=W+dw;
  %W=W/sum(abs(W));
  
[k,m]
end

for t=1:7
   outneuron(t,k)= G(sum(SensResp(t,r0,rm,sigma).*W),a,b);
end


W_Updated(k,:)=W;
% 3 Vs 5
for f=1:10
LearningTest1(f)=(G(sum(SensResp(5,r0,rm,sigma).*W),a,b))-(G(sum(SensResp(4,r0,rm,sigma).*W),a,b));%/(sum(TC(f,2,r0,rm,sigma).*W)-sum(TC(f,6,r0,rm,sigma).*W)); % Test 3 vs 4 
% 0 vs 2
ConflictTest1(f)=(G(sum(SensResp(4,r0,rm,sigma).*W),a,b))-(G(sum(SensResp(1,r0,rm,sigma).*W),a,b));%/(sum(TC(f,2,r0,rm,sigma).*W)-sum(TC(f,6,r0,rm,sigma).*W)); % Test 0 vs 3 
% 0 vs 1
TransferTest1(f)=(G(sum(SensResp(6,r0,rm,sigma).*W),a,b))-(G(sum(SensResp(3,r0,rm,sigma).*W),a,b));%/(sum(TC(f,2,r0,rm,sigma).*W)-sum(TC(f,6,r0,rm,sigma).*W)); % Test 2 vs 5 
end

LearningTest_e1_2(k)=sum(LearningTest1>0)/10;
ConflictTest_e1_2(k)=sum(ConflictTest1>0)/10;
TransferTest_e1_2(k)=sum(TransferTest1>0)/10;

%W_updated(k,:)=W;
end


figure
subplot(1,3,1)
errorbar(mean(outneuron'),std(outneuron')/sqrt(n_trials))

subplot(1,3,3)
meanTest_Exp1_2=[mean(LearningTest_e1_2) mean(ConflictTest_e1_2) mean(TransferTest_e1_2)];
SETest_Exp1_2=[std(LearningTest_e1_2) std(ConflictTest_e1_2) std(TransferTest_e1_2)]/sqrt(n_trials);
hold on
bar(meanTest_Exp1_2)
errorbar(meanTest_Exp1_2,SETest_Exp1_2)
set(gca,'xTick',[1 2 3],'xTickLabel',{'Learning test[3 4]','Conflict test[0 3]','Transfer test[2 5]'})
ylabel('Performance (%)')
title('Model Performance in Tests')

subplot(1,3,2)
errorbar(mean(W_Updated),std(W_Updated)/sqrt(n_trials))
set(gca,'xTick',[1 2 3 4 5 6 7],'xTickLabel',{'0','1','2','3','4','5','6'})
title('optimal Synapic weights')


%%%%% Final Figure

MEANTEST=[meanTest_Exp1_1;meanTest_Exp1_2];
SETEST=[SETest_Exp1_1;SETest_Exp1_2];

[bar_xtick,hb,he]=errorbar_groups(MEANTEST,SETEST);
legend('Less than', 'Greater than')
% set(gca,'xTick',[1.1 1.8 3.1 3.8 5.1 5.8],'xTickLabel',{' Less','More',' Less','More','Less','More'})
set(gca,'xTick',[1.5 3.5 5.5],'xTickLabel',{'Learning test','Conflict test',' Transfer tset'})
ylabel('Proportion of correct choices(%)')


%%   Experiment 2 (less than) % trained to [2 5] and tested (Figure 3C)
% this section, the model is traineid to stimuli with 2,3,4 and 5 elements.
% Then this is examined with the conflict and transfer test by showing the
% blank stimuli.


% Training protocol
Stimulipair=[2 2 2 3 3 4 ;...
             3 4 5 4 5 5 ];    
                 
n_presentation=10;
StimuliPair=[];
for tNP=1:n_presentation
StimuliPair=[StimuliPair,Stimulipair];
end

% StimuliPair=[2 2 2 3 3 4 2 2 2 3 3 4 2 2 2 3 3 4 2 2 2 3 3 4 2 2 2 3 3 4 2 2 2 3 3 4;...
%              3 4 5 4 5 5 3 4 5 4 5 5 3 4 5 4 5 5 3 4 5 4 5 5 3 4 5 4 5 5 3 4 5 4 5 5];
 

% a=.06;
% b=55;
% r0=50;
% rm=100;
% sigma=2.9;

% a=.05;
% b=60;
% r0=50;
% rm=100;
% sigma=2.8;

% a=.05;
% b=60;
% r0=50;
% rm=100;
% sigma=2;

a=.06;
b=70; % change the baseline actvity of the post neurons
r0=40;
rm=100;
sigma=2.8;

% Sigmoid function
G =@ (x,a,b) 110./(1+exp(-a*(x-b)))+4*randn;
Gp=@ (x,a,b) 110*a*exp(-a*(x-b))./(1+exp(-a*(x-b))).^2;  

f=1:7;
eta=1e-3; % Learning term

 n_trials=50;     
for k=1:n_trials
% Initial synaptic weights 
W=rand(1,7);
W=W/sum(abs(W));

 stimuli2=StimuliPair(:,randperm(size(StimuliPair,2)));   

for m=1:size(stimuli2,2)
    kp=stimuli2(1,m)+1;
    kn=stimuli2(2,m)+1;
   
 dw=eta*(SensResp(kp,r0,rm,sigma)*Gp(sum(SensResp(kp,r0,rm,sigma).*W),a,b)-SensResp(kn,r0,rm,sigma)*Gp(sum(SensResp(kn,r0,rm,sigma).*W),a,b)); % best 
 
 W=W+dw;
  %W=W/sum(abs(W));
  
% dw1=eta*TC(f,kp,r0,rm,sigma).*Gp(sum(TC(f,kp,r0,rm,sigma).*W)).*(1-G(sum(TC(f,kp,r0,rm,sigma).*W)));
% dw1=eta*TC(f,kn,r0,rm,sigma).*Gp(sum(TC(f,kp,r0,rm,sigma).*W)).*(0-G(sum(TC(f,kn,r0,rm,sigma).*W)));
% W=W/sum(abs(W));

[k,m]
end

for t=1:7
   outneuron(t,k)= G(sum(SensResp(t,r0,rm,sigma).*W),a,b);
end


W_Updated(k,:)=W;
% 3 Vs 5
for f=1:10
LearningTest1(f)=(G(sum(SensResp(4,r0,rm,sigma).*W),a,b))-(G(sum(SensResp(6,r0,rm,sigma).*W),a,b));%/(sum(TC(f,2,r0,rm,sigma).*W)-sum(TC(f,6,r0,rm,sigma).*W)); % Test 3 vs 4 
% 0 vs 2
ConflictTest1(f)=(G(sum(SensResp(1,r0,rm,sigma).*W),a,b))-(G(sum(SensResp(3,r0,rm,sigma).*W),a,b));%/(sum(TC(f,2,r0,rm,sigma).*W)-sum(TC(f,6,r0,rm,sigma).*W)); % Test 0 vs 3 
% 0 vs 1
TransferTest1(f)=(G(sum(SensResp(1,r0,rm,sigma).*W),a,b))-(G(sum(SensResp(2,r0,rm,sigma).*W),a,b));%/(sum(TC(f,2,r0,rm,sigma).*W)-sum(TC(f,6,r0,rm,sigma).*W)); % Test 2 vs 5 
end

LearningTest(k)=sum(LearningTest1>0)/10;
ConflictTest(k)=sum(ConflictTest1>0)/10;
TransferTest(k)=sum(TransferTest1>0)/10;

%W_updated(k,:)=W;
end

figure
subplot(1,3,1)
errorbar(mean(outneuron'),std(outneuron')/sqrt(n_trials))

subplot(1,3,3)
meanTest_Exp1=[mean(LearningTest) mean(ConflictTest) mean(TransferTest)];
SETest_Exp1=[std(LearningTest) std(ConflictTest) std(TransferTest)]/sqrt(n_trials);
hold on
bar(meanTest_Exp1)
errorbar(meanTest_Exp1,SETest_Exp1)
set(gca,'xTick',[1 2 3],'xTickLabel',{'Learning test[3 5]','Conflict test[0 2','Transfer test[0 1]'})
ylabel('Difference Firing rate')
title('Model Performance in Tests')

subplot(1,3,2)
errorbar(mean(W_Updated),std(W_Updated)/sqrt(n_trials))
set(gca,'xTick',[1 2 3 4 5 6 7],'xTickLabel',{'0','1','2','3','4','5','6'})
title('optimal Synapic weights')



[bar_xtick,hb,he]=errorbar_groups(meanTest_Exp1,SETest_Exp1);
%legend('Less than', 'Greater than')
% set(gca,'xTick',[1.1 1.8 3.1 3.8 5.1 5.8],'xTickLabel',{' Less','More',' Less','More','Less','More'})
set(gca,'xTick',[1 2 3],'xTickLabel',{'Learning test','Conflict test',' Transfer tset'})
ylabel('Proportion of correct choices(%)')
ylim([0 1])


%% Experiment 3 (Figure 3e)
% this section the model recognise stimuli with zero elements as the lower end of a continuum



% Training protocol
Stimulipair=[1 1 1 1 1 1 2 2 2 2 3 3 3 4 4 5;...
             2 3 4 5 6 6 3 4 5 6 4 5 6 5 6 6];    
                 
n_presentation=10;
StimuliPair=[];
for tNP=1:n_presentation
StimuliPair=[StimuliPair,Stimulipair];
end

a=.06;
b=70; % change the baseline actvity of the post neurons
r0=40;
rm=100;
sigma=3.0;

% Sigmoid function
G =@ (x,a,b) 110./(1+exp(-a*(x-b)))+3*randn;
Gp=@ (x,a,b) 110*a*exp(-a*(x-b))./(1+exp(-a*(x-b))).^2;  

f=1:7;
eta=1e-3; % Learning term

 n_trials=50;     
for k=1:n_trials
% Initial synaptic weights 
W=rand(1,7);
W=W/sum(abs(W));

 stimuli2=StimuliPair(:,randperm(size(StimuliPair,2)));   

for m=1:size(stimuli2,2)
    kp=stimuli2(1,m)+1;
    kn=stimuli2(2,m)+1;
   
 dw=eta*(SensResp(kp,r0,rm,sigma)*Gp(sum(SensResp(kp,r0,rm,sigma).*W),a,b)-SensResp(kn,r0,rm,sigma)*Gp(sum(SensResp(kn,r0,rm,sigma).*W),a,b)); % best 
 
  W=W+dw;

[k,m]
end


for t=1:7
   outneuron(t,k)= G(sum(SensResp(t,r0,rm,sigma).*W),a,b);
end


W_Updated(k,:)=W;
% 3 Vs 5
for f=1:10
LearningTest1(f)=(G(sum(SensResp(1,r0,rm,sigma).*W),a,b))-(G(sum(SensResp(2,r0,rm,sigma).*W),a,b));%/(sum(TC(f,2,r0,rm,sigma).*W)-sum(TC(f,6,r0,rm,sigma).*W)); % Test 3 vs 4 
LearningTest2(f)=(G(sum(SensResp(1,r0,rm,sigma).*W),a,b))-(G(sum(SensResp(3,r0,rm,sigma).*W),a,b));
LearningTest3(f)=(G(sum(SensResp(1,r0,rm,sigma).*W),a,b))-(G(sum(SensResp(4,r0,rm,sigma).*W),a,b));
LearningTest4(f)=(G(sum(SensResp(1,r0,rm,sigma).*W),a,b))-(G(sum(SensResp(5,r0,rm,sigma).*W),a,b));
LearningTest5(f)=(G(sum(SensResp(1,r0,rm,sigma).*W),a,b))-(G(sum(SensResp(6,r0,rm,sigma).*W),a,b));
LearningTest6(f)=(G(sum(SensResp(1,r0,rm,sigma).*W),a,b))-(G(sum(SensResp(7,r0,rm,sigma).*W),a,b));
end


LT1(k)=sum(LearningTest1>0)/10;
LT2(k)=sum(LearningTest2>0)/10;
LT3(k)=sum(LearningTest3>0)/10;
LT4(k)=sum(LearningTest4>0)/10;
LT5(k)=sum(LearningTest5>0)/10;
LT6(k)=sum(LearningTest6>0)/10;


%W_updated(k,:)=W;
end

figure
subplot(1,3,1)
errorbar(mean(outneuron'),std(outneuron')/sqrt(n_trials))

subplot(1,3,3)
meanTest_Exp3=[mean(LT1) mean(LT2) mean(LT3) mean(LT4) mean(LT5) mean(LT6)];
SETest_Exp3=[std(LT1) std(LT2) std(LT3) std(LT4) std(LT5) std(LT6)]/sqrt(n_trials);
hold on
bar(meanTest_Exp3)
errorbar(meanTest_Exp3,SETest_Exp3)
set(gca,'xTick',[1 2 3],'xTickLabel',{'Learning test[3 5]','Conflict test[0 2','Transfer test[0 1]'})
ylabel('Difference Firing rate')
title('Model Performance in Tests')

subplot(1,3,2)
errorbar(mean(W_Updated),std(W_Updated)/sqrt(n_trials))
set(gca,'xTick',[1 2 3 4 5 6 7],'xTickLabel',{'0','1','2','3','4','5','6'})
title('optimal Synapic weights')


figure
bar(meanTest_Exp3)
hold on
errorbar(meanTest_Exp3,SETest_Exp3)
set(gca,'xTick',[1 2 3 4 5 6],'xTickLabel',{'0 V 1','0 V 2','0 V 3','0 V 4','0 V 5','0 V 6'})
ylabel('Proportion of choicies for the blank(%)')
ylim([0 1])

%% Experiment 4 --- Trainted to 2 vs 3 and tested with zero and 2 or 3 



a=.06;
b=70; % change the baseline actvity of the post neurons
r0=40;
rm=100;
sigma=.8;

% Sigmoid function
G =@ (x,a,b) 110./(1+exp(-a*(x-b)))+4*randn;
Gp=@ (x,a,b) 110*a*exp(-a*(x-b))./(1+exp(-a*(x-b))).^2;  

f=1:7;
eta=1e-3; % Learning term
 n_trials=50; 
 
 
 
%%%%%%%%%%%%%%%%%%%%%%
%%%% Less than 

% Training protocol
Stimulipair=[ 2 2 2 2 2 ;...
              3 3 3 3 3 ];    
                 
n_presentation=10;
StimuliPair=[];
for tNP=1:n_presentation
StimuliPair=[StimuliPair,Stimulipair];
end

 
for k=1:n_trials
% Initial synaptic weights 
W=rand(1,7);
W=W/sum(abs(W));

 stimuli2=StimuliPair(:,randperm(size(StimuliPair,2)));   

for m=1:size(stimuli2,2)
    kp=stimuli2(1,m)+1;
    kn=stimuli2(2,m)+1;
   
 dw=eta*(SensResp(kp,r0,rm,sigma)*Gp(sum(SensResp(kp,r0,rm,sigma).*W),a,b)-SensResp(kn,r0,rm,sigma)*Gp(sum(SensResp(kn,r0,rm,sigma).*W),a,b)); % best 
 
 W=W+dw;
  %W=W/sum(abs(W));
  
[k,m]
end

for t=1:7
   outneuron(t,k)= G(sum(SensResp(t,r0,rm,sigma).*W),a,b);
end


W_Updated(k,:)=W;
% 3 Vs 5
for f=1:10
LearningTest4(f)=(G(sum(SensResp(3,r0,rm,sigma).*W),a,b))-(G(sum(SensResp(4,r0,rm,sigma).*W),a,b));%/(sum(TC(f,2,r0,rm,sigma).*W)-sum(TC(f,6,r0,rm,sigma).*W)); % Test 3 vs 4 
% 0 vs 2
ConflictTest4(f)=(G(sum(SensResp(4,r0,rm,sigma).*W),a,b))-(G(sum(SensResp(1,r0,rm,sigma).*W),a,b));%/(sum(TC(f,2,r0,rm,sigma).*W)-sum(TC(f,6,r0,rm,sigma).*W)); % Test 0 vs 3 
% 0 vs 1
TransferTest4(f)=(G(sum(SensResp(3,r0,rm,sigma).*W),a,b))-(G(sum(SensResp(1,r0,rm,sigma).*W),a,b));%/(sum(TC(f,2,r0,rm,sigma).*W)-sum(TC(f,6,r0,rm,sigma).*W)); % Test 2 vs 5 
end

LearningTest_e4_1(k)=sum(LearningTest4>0)/10;
ConflictTest_e4_1(k)=sum(ConflictTest4>0)/10;
TransferTest_e4_1(k)=sum(TransferTest4>0)/10;

%W_updated(k,:)=W;
end

figure
subplot(1,3,1)
errorbar(mean(outneuron'),std(outneuron')/sqrt(n_trials))

subplot(1,3,3)
meanTest_Exp4_1=[mean(LearningTest_e4_1) mean(ConflictTest_e4_1) mean(TransferTest_e4_1)];
SETest_Exp4_1=[std(LearningTest_e4_1) std(ConflictTest_e4_1) std(TransferTest_e4_1)]/sqrt(n_trials);
hold on
bar(meanTest_Exp4_1)
errorbar(meanTest_Exp4_1,SETest_Exp4_1)
set(gca,'xTick',[1 2 3],'xTickLabel',{'Learning test[3 4]','Conflict test[0 3]','Transfer test[2 5]'})
ylabel('Performance (%)')
title('Model Performance in Tests')

subplot(1,3,2)
errorbar(mean(W_Updated),std(W_Updated)/sqrt(n_trials))
set(gca,'xTick',[1 2 3 4 5 6 7],'xTickLabel',{'0','1','2','3','4','5','6'})
title('optimal Synapic weights')

%%   Experiment 1 (less than) % trained to [1 5] except the pair (2 3) and tested 

a=.06;
b=70; % change the baseline actvity of the post neurons
r0=40;
rm=100;
sigma=2.8;

% Sigmoid function
G =@ (x,a,b) 110./(1+exp(-a*(x-b)))+4*randn;
Gp=@ (x,a,b) 110*a*exp(-a*(x-b))./(1+exp(-a*(x-b))).^2;  

f=1:7;
eta=1e-3; % Learning term
 n_trials=50; 
 
 
 
%%%%%%%%%%%%%%%%%%%%%%
%%%% Less than 

% Training protocol
Stimulipair=[ 1 1 1 1 2 2 3 3 4;...
              2 3 4 5 4 5 4 5 5];    
                 
n_presentation=10;
StimuliPair=[];
for tNP=1:n_presentation
StimuliPair=[StimuliPair,Stimulipair];
end

 
for k=1:n_trials
% Initial synaptic weights 
W=rand(1,7);
W=W/sum(abs(W));

 stimuli2=StimuliPair(:,randperm(size(StimuliPair,2)));   

for m=1:size(stimuli2,2)
    kp=stimuli2(1,m)+1;
    kn=stimuli2(2,m)+1;
   
 dw=eta*(SensResp(kp,r0,rm,sigma)*Gp(sum(SensResp(kp,r0,rm,sigma).*W),a,b)-SensResp(kn,r0,rm,sigma)*Gp(sum(SensResp(kn,r0,rm,sigma).*W),a,b)); % best 
 
 W=W+dw;
  %W=W/sum(abs(W));
  
[k,m]
end

for t=1:7
   outneuron(t,k)= G(sum(SensResp(t,r0,rm,sigma).*W),a,b);
end


W_Updated(k,:)=W;
% 3 Vs 5
for f=1:10
LearningTest5(f)=(G(sum(SensResp(3,r0,rm,sigma).*W),a,b))-(G(sum(SensResp(4,r0,rm,sigma).*W),a,b));%/(sum(TC(f,2,r0,rm,sigma).*W)-sum(TC(f,6,r0,rm,sigma).*W)); % Test 3 vs 4 
% 0 vs 2
ConflictTest5(f)=(G(sum(SensResp(1,r0,rm,sigma).*W),a,b))-(G(sum(SensResp(4,r0,rm,sigma).*W),a,b));%/(sum(TC(f,2,r0,rm,sigma).*W)-sum(TC(f,6,r0,rm,sigma).*W)); % Test 0 vs 3 

end

LearningTest_e5_1(k)=sum(LearningTest5>0)/10;
ConflictTest_e5_1(k)=sum(ConflictTest5>0)/10;


%W_updated(k,:)=W;
end

figure
subplot(1,3,1)
errorbar(mean(outneuron'),std(outneuron')/sqrt(n_trials))

subplot(1,3,3)
meanTest_Exp5_1=[mean(LearningTest_e5_1) mean(ConflictTest_e5_1)];
SETest_Exp5_1=[std(LearningTest_e5_1) std(ConflictTest_e5_1)]/sqrt(n_trials);
hold on
bar(meanTest_Exp5_1)
errorbar(meanTest_Exp5_1,SETest_Exp5_1)
set(gca,'xTick',[1 2],'xTickLabel',{'Learning test[3 4]','Conflict test[0 3]'})
ylabel('Performance (%)')
title('Model Performance in Tests')

subplot(1,3,2)
errorbar(mean(W_Updated),std(W_Updated)/sqrt(n_trials))
set(gca,'xTick',[1 2 3 4 5 6 7],'xTickLabel',{'0','1','2','3','4','5','6'})
title('optimal Synapic weights')



%%%%%%%%%%%%%%%%%%%%%%
%%%% greater than 

% Training protocol
Stimulipair=[ 1 1 1 1 2 2 3 3 4;...
              2 3 4 5 4 5 4 5 5];    
                 
n_presentation=10;
StimuliPair=[];
for tNP=1:n_presentation
StimuliPair=[StimuliPair,Stimulipair];
end

 
for k=1:n_trials
% Initial synaptic weights 
W=rand(1,7);
W=W/sum(abs(W));

 stimuli2=StimuliPair(:,randperm(size(StimuliPair,2)));   

for m=1:size(stimuli2,2)
    kp=stimuli2(2,m)+1;
    kn=stimuli2(1,m)+1;
   
 dw=eta*(SensResp(kp,r0,rm,sigma)*Gp(sum(SensResp(kp,r0,rm,sigma).*W),a,b)-SensResp(kn,r0,rm,sigma)*Gp(sum(SensResp(kn,r0,rm,sigma).*W),a,b)); % best 
 
 W=W+dw;
  %W=W/sum(abs(W));
  
[k,m]
end

for t=1:7
   outneuron(t,k)= G(sum(SensResp(t,r0,rm,sigma).*W),a,b);
end


W_Updated(k,:)=W;
% 3 Vs 5
for f=1:10
LearningTest5(f)=(G(sum(SensResp(4,r0,rm,sigma).*W),a,b))-(G(sum(SensResp(3,r0,rm,sigma).*W),a,b));%/(sum(TC(f,2,r0,rm,sigma).*W)-sum(TC(f,6,r0,rm,sigma).*W)); % Test 3 vs 4 
% 0 vs 2
ConflictTest5(f)=(G(sum(SensResp(4,r0,rm,sigma).*W),a,b))-(G(sum(SensResp(1,r0,rm,sigma).*W),a,b));%/(sum(TC(f,2,r0,rm,sigma).*W)-sum(TC(f,6,r0,rm,sigma).*W)); % Test 0 vs 3 

end

LearningTest_e5_2(k)=sum(LearningTest5>0)/10;
ConflictTest_e5_2(k)=sum(ConflictTest5>0)/10;


%W_updated(k,:)=W;
end

figure
subplot(1,3,1)
errorbar(mean(outneuron'),std(outneuron')/sqrt(n_trials))

subplot(1,3,3)
meanTest_Exp5_2=[mean(LearningTest_e5_2) mean(ConflictTest_e5_2)];
SETest_Exp5_2=[std(LearningTest_e5_2) std(ConflictTest_e5_2)]/sqrt(n_trials);
hold on
bar(meanTest_Exp5_2)
errorbar(meanTest_Exp5_2,SETest_Exp5_2)
set(gca,'xTick',[1 2],'xTickLabel',{'Learning test[3 4]','Conflict test[0 3]'})
ylabel('Performance (%)')
title('Model Performance in Tests')

subplot(1,3,2)
errorbar(mean(W_Updated),std(W_Updated)/sqrt(n_trials))
set(gca,'xTick',[1 2 3 4 5 6 7],'xTickLabel',{'0','1','2','3','4','5','6'})
title('optimal Synapic weights')



%%%%% Final Figure

MEANTEST5=[meanTest_Exp5_1;meanTest_Exp5_2];
SETEST5=[SETest_Exp5_1;SETest_Exp5_2];

[bar_xtick,hb,he]=errorbar_groups(MEANTEST5,SETEST5);
legend('Less than', 'Greater than')
% set(gca,'xTick',[1.1 1.8 3.1 3.8 5.1 5.8],'xTickLabel',{' Less','More',' Less','More','Less','More'})
set(gca,'xTick',[1.5 3.5],'xTickLabel',{'Learning test','Conflict test',})
ylabel('Proportion of correct choices(%)')

%% Experiment 3 without learning 
 % to evaluate the Plasticity rule in the model 


% Training protocol
Stimulipair=[1 1 1 1 1 1 2 2 2 2 3 3 3 4 4 5;...
             2 3 4 5 6 6 3 4 5 6 4 5 6 5 6 6];    
                 
n_presentation=10;
StimuliPair=[];
for tNP=1:n_presentation
StimuliPair=[StimuliPair,Stimulipair];
end

% 
a=.06;
b=70; % change the baseline actvity of the post neurons
r0=40;
rm=100;
sigma=2.8;

% Sigmoid function
G =@ (x,a,b) 110./(1+exp(-a*(x-b)))+3*randn;
Gp=@ (x,a,b) 110*a*exp(-a*(x-b))./(1+exp(-a*(x-b))).^2;  

f=1:7;
eta=1e-3; % Learning term

 n_trials=50;     
for k=1:n_trials
% Initial synaptic weights 
W=randn(1,7);
W=W/sum(abs(W));

 stimuli2=StimuliPair(:,randperm(size(StimuliPair,2)));   

for m=1:size(stimuli2,2)
    kp=stimuli2(1,m)+1;
    kn=stimuli2(2,m)+1;
   
 dw=eta*(SensResp(kp,r0,rm,sigma)*Gp(sum(SensResp(kp,r0,rm,sigma).*W),a,b)-SensResp(kn,r0,rm,sigma)*Gp(sum(SensResp(kn,r0,rm,sigma).*W),a,b)); % best 
 

[k,m]
end


for t=1:7
   outneuron(t,k)= G(sum(SensResp(t,r0,rm,sigma).*W),a,b);
end


W_Updated(k,:)=W;
% 3 Vs 5
l_test=20;
for f=1:l_test
LearningTest1(f)=(G(sum(SensResp(1,r0,rm,sigma).*W),a,b))-(G(sum(SensResp(2,r0,rm,sigma).*W),a,b));%/(sum(TC(f,2,r0,rm,sigma).*W)-sum(TC(f,6,r0,rm,sigma).*W)); % Test 3 vs 4 
LearningTest2(f)=(G(sum(SensResp(1,r0,rm,sigma).*W),a,b))-(G(sum(SensResp(3,r0,rm,sigma).*W),a,b));
LearningTest3(f)=(G(sum(SensResp(1,r0,rm,sigma).*W),a,b))-(G(sum(SensResp(4,r0,rm,sigma).*W),a,b));
LearningTest4(f)=(G(sum(SensResp(1,r0,rm,sigma).*W),a,b))-(G(sum(SensResp(5,r0,rm,sigma).*W),a,b));
LearningTest5(f)=(G(sum(SensResp(1,r0,rm,sigma).*W),a,b))-(G(sum(SensResp(6,r0,rm,sigma).*W),a,b));
LearningTest6(f)=(G(sum(SensResp(1,r0,rm,sigma).*W),a,b))-(G(sum(SensResp(7,r0,rm,sigma).*W),a,b));
end


LT1(k)=sum(LearningTest1>0)/l_test;
LT2(k)=sum(LearningTest2>0)/l_test;
LT3(k)=sum(LearningTest3>0)/l_test;
LT4(k)=sum(LearningTest4>0)/l_test;
LT5(k)=sum(LearningTest5>0)/l_test;
LT6(k)=sum(LearningTest6>0)/l_test;

%W_updated(k,:)=W;
end

figure
subplot(1,3,1)
errorbar(mean(outneuron'),std(outneuron')/sqrt(n_trials))

subplot(1,3,3)
meanTest_Exp3=[mean(LT1) mean(LT2) mean(LT3) mean(LT4) mean(LT5) mean(LT6)];
SETest_Exp3=[std(LT1) std(LT2) std(LT3) std(LT4) std(LT5) std(LT6)]/sqrt(n_trials);
hold on
bar(meanTest_Exp3)
errorbar(meanTest_Exp3,SETest_Exp3)
set(gca,'xTick',[1 2 3],'xTickLabel',{'Learning test[3 5]','Conflict test[0 2','Transfer test[0 1]'})
ylabel('Difference Firing rate')
title('Model Performance in Tests')

subplot(1,3,2)
errorbar(mean(W_Updated),std(W_Updated)/sqrt(n_trials))
set(gca,'xTick',[1 2 3 4 5 6 7],'xTickLabel',{'0','1','2','3','4','5','6'})
title('optimal Synapic weights')


figure
bar(meanTest_Exp3)
hold on
errorbar(meanTest_Exp3,SETest_Exp3)
set(gca,'xTick',[1 2 3 4 5 6],'xTickLabel',{'0 V 1','0 V 1','0 V 3','0 V 4','0 V 5','0 V 6'})
ylabel('Proportion of choicies for the blank(%)')
ylim([0 1])

